@@ -88,19 +88,6 @@ gather <- function(data, key = "key", value = "value", ...,
   UseMethod("gather")
 }
 #' @export
-gather.default <- function(data, key = "key", value = "value", ...,
-                           na.rm = FALSE, convert = FALSE,
-                           factor_key = FALSE) {
-  gather_(data,
-    key_col = compat_as_lazy(enquo(key)),
-    value_col = compat_as_lazy(enquo(value)),
-    ...,
-    na.rm = na.rm,
-    convert = convert,
-    factor_key = factor_key
-  )
-}
-#' @export
 gather.data.frame <- function(data, key = "key", value = "value", ...,
                               na.rm = FALSE, convert = FALSE,
                               factor_key = FALSE) {
@@ -113,22 +100,23 @@ gather.data.frame <- function(data, key = "key", value = "value", ...,
   } else {
     gather_vars <- unname(tidyselect::vars_select(names(data), !!! quos))
   }
+
   if (is_empty(gather_vars)) {
     return(data)
   }
 
   gather_idx <- match(gather_vars, names(data))
-  if (anyNA(gather_idx)) {
-    missing_cols <- paste(gather_vars[is.na(gather_idx)], collapse = ", ")
-    abort(glue("Unknown column names: ", missing_cols))
-  }
   id_idx <- setdiff(seq_along(data), gather_idx)
 
+  dup_indx <- match(c(key_var, value_var), names(data))
+  id_idx <- setdiff(id_idx, dup_indx)
+
   ## Get the attributes if common, NULL if not.
-  args <- normalize_melt_arguments(data, gather_idx, factorsAsStrings = TRUE)
+  args <- normalize_melt_arguments(data, gather_idx)
   valueAsFactor <- "factor" %in% class(args$attr_template)
 
-  out <- melt_dataframe(data,
+  out <- melt_dataframe(
+    data,
     id_idx - 1L,
     gather_idx - 1L,
     as.character(key_var),
@@ -154,8 +142,7 @@ gather.data.frame <- function(data, key = "key", value = "value", ...,
 # Functions from reshape2 -------------------------------------------------
 
 ## Get the attributes if common, NULL if not.
-normalize_melt_arguments <- function(data, measure.ind, factorsAsStrings) {
-
+normalize_melt_arguments <- function(data, measure.ind) {
   measure.attributes <- map(measure.ind, function(i) {
     attributes(data[[i]])
   })
@@ -168,26 +155,22 @@ normalize_melt_arguments <- function(data, measure.ind, factorsAsStrings) {
   } else {
     warn(glue(
       "attributes are not identical across measure variables;
-       they will be dropped"))
+       they will be dropped"
+    ))
     attr_template <- NULL
   }
 
-  if (!factorsAsStrings && !measure.attrs.equal) {
-    warn("cannot avoid coercion of factors when measure attributes not identical")
-    factorsAsStrings <- TRUE
-  }
-
   ## If we are going to be coercing any factors to strings, we don't want to
   ## copy the attributes
   any.factors <- any(map_lgl(measure.ind, function(i) is.factor(data[[i]])))
 
-  if (factorsAsStrings && any.factors) {
+  if (any.factors) {
     attr_template <- NULL
   }
 
   list(
     attr_template = attr_template,
-    factorsAsStrings = factorsAsStrings
+    factorsAsStrings = TRUE
   )
 }
 
@@ -198,34 +181,3 @@ all_identical <- function(xs) {
   }
   TRUE
 }
-
-
-#' @rdname deprecated-se
-#' @inheritParams gather
-#' @param key_col,value_col Strings giving names of key and value columns to
-#'   create.
-#' @param gather_cols Character vector giving column names to be gathered into
-#'   pair of key-value columns.
-#' @keywords internal
-#' @export
-gather_ <- function(data, key_col, value_col, gather_cols, na.rm = FALSE,
-                    convert = FALSE, factor_key = FALSE) {
-  UseMethod("gather_")
-}
-#' @export
-gather_.data.frame <- function(data, key_col, value_col, gather_cols,
-                               na.rm = FALSE, convert = FALSE,
-                               factor_key = FALSE) {
-  key_col <- sym(key_col)
-  value_col <- sym(value_col)
-  gather_cols <- syms(gather_cols)
-
-  gather(data,
-    key = !! key_col,
-    value = !! value_col,
-    !!! gather_cols,
-    na.rm = na.rm,
-    convert = convert,
-    factor_key = factor_key
-  )
-}
